@@ -40,14 +40,28 @@ import yaml
 import textwrap
 import locale
 import importlib.util
-import platform
+import re
 
 from tempfile import TemporaryDirectory
 from shutil import move, rmtree
 from distutils.version import StrictVersion
 
 import mlhub.utils as utils
-from mlhub.constants import MLINIT, DESC_YAML, DESC_YML, APP, APPX, CMD, HUB_PATH, EXT_MLM, EXT_AIPK, META_YAML, README, META_YML, DEBUG
+from mlhub.constants import (
+    MLINIT,
+    DESC_YAML,
+    DESC_YML,
+    APP,
+    APPX,
+    CMD,
+    HUB_PATH,
+    EXT_MLM,
+    EXT_AIPK,
+    META_YAML,
+    README,
+    META_YML,
+    DEBUG,
+)
 
 # The commands are implemented here in a logical order with each
 # command providing a suggesting of the following command.
@@ -77,9 +91,7 @@ def list_available(args):
     # Suggest next step.
     
     if not args.quiet:
-        msg = "\nInstall a model with:\n\n  $ {} install <model>\n"
-        msg = msg.format(CMD)
-        print(msg)
+        utils.print_next_step('available')
 
 #------------------------------------------------------------------------
 # INSTALLED
@@ -88,27 +100,28 @@ def list_available(args):
 def list_installed(args):
     """List the installed models."""
 
-    # Find installed models.
+    # Find installed models, ignoring special folders like R.
 
     if os.path.exists(MLINIT):
-        msg = "in '{}'.".format(MLINIT)
+        msg = " in '{}'.".format(MLINIT)
         models = [f for f in os.listdir(MLINIT)
-                  if os.path.isdir(os.path.join(MLINIT, f))]
+                  if os.path.isdir(os.path.join(MLINIT, f)) and f != "R"]
     else:
-        msg = "since '{}' does not exist.".format(MLINIT)
+        msg = ". '{}' does not exist.".format(MLINIT)
         models = []
 
+    models.sort()
+        
     # Report on how many models we found installed.
         
     mcnt = len(models)
     plural = "s"
     if mcnt == 1: plural = ""
-    print("Found {} model{} installed {}".format(mcnt, plural, msg))
+    print("Found {} model{} installed{}".format(mcnt, plural, msg))
 
     # Report on each of the installed models.
         
     if mcnt > 0: print("")
-    
     for p in models:
         entry = utils.load_description(p)
         utils.print_meta_line(entry)
@@ -116,21 +129,25 @@ def list_installed(args):
     # Suggest next step.
     
     if not args.quiet:
-        msg = "\nRun a model's demonstration script with:\n\n  $ {} demo <model>\n"
-        msg = msg.format(CMD)
-        print(msg)
+        if mcnt > 0:
+            utils.print_next_step('installed', scenario='exist')
+        else:
+            utils.print_next_step('installed', scenario='none')
 
 #-----------------------------------------------------------------------
 # INSTALL
+#------------------------------------------------------------------------
 
 def install_model(args):
     """Install a model."""
 
     # Setup.
 
+    url = None
+
     # Identify if it is a local file name to install.
     
-    if args.model.endswith(EXT_MLM):
+    if args.model.endswith(EXT_MLM) and not re.findall('http[s]?:', args.model):
 
         # Identify the local mlm file to install.
 
@@ -146,20 +163,38 @@ def install_model(args):
 
     else:
 
-        # Obtain the repository meta data from Packages.yaml.
+        if re.findall('http[s]?:', args.model):
 
-        model = args.model
+            # A specific URL was provided.
+            
+            url = args.model
+            model = url.split("/")[-1].split("_")[0]
+
+            # Check that the URL exists.
+            
+            r = requests.head(url)
+            if not r.status_code == requests.codes.ok:
+                msg = "{}the url '{}' was not found."
+                msg = msg.format(APPX, url)
+                print(msg, file=sys.stderr)
+                sys.exit(1)
+            
+        else:
+            
+            # Or obtain the repository meta data from Packages.yaml.
+
+            model = args.model
         
-        mlhub = utils.get_repo(args.mlhub)
-        meta  = utils.get_repo_meta_data(mlhub)
+            mlhub = utils.get_repo(args.mlhub)
+            meta  = utils.get_repo_meta_data(mlhub)
 
-        # Find the first matching entry in the meta data.
+            # Find the first matching entry in the meta data.
 
-        url = None
-        for entry in meta:
-            if model == entry["meta"]["name"]:
-                url = mlhub + entry["meta"]["filename"]
-                break
+            url = None
+            for entry in meta:
+                if model == entry["meta"]["name"]:
+                    url = mlhub + entry["meta"]["filename"]
+                    break
 
         # If not found suggest how a model might be installed.
 
@@ -172,7 +207,6 @@ def install_model(args):
                 msg = msg.format(CMD)
                 print(msg, file=sys.stderr)
             sys.exit(1)
-
             
         if args.debug: print(DEBUG + "model file url is: " + url)
 
@@ -193,24 +227,6 @@ def install_model(args):
         local   = os.path.join(init, mlmfile)
         path    = os.path.join(init, model)
 
-        # Informative message about the model location and size.
-
-        if not args.quiet: print("Model " + url + "\n")
-        meta = requests.head(url)
-        dsize = "{:,}".format(int(meta.headers.get("content-length")))
-        if not args.quiet: print("Downloading '{}' ({} bytes) ...\n".
-                                 format(mlmfile, dsize))
-
-        # Download the archive from the URL.
-
-        try:
-            urllib.request.urlretrieve(url, local)
-        except urllib.error.HTTPError as error:
-            msg = "{}'{}' {}."
-            msg = msg.format(APPX, url, error.reason.lower())
-            print(msg, file=sys.stderr)
-            sys.exit(1)
-
     # Check if model is already installed.
         
     if os.path.exists(path):
@@ -234,7 +250,29 @@ def install_model(args):
             print(msg)
         rmtree(path)
         print()
-    
+
+    # Download the model now if not a local file.
+        
+    if not url is None:
+
+        # Informative message about the model location and size.
+
+        if not args.quiet: print("Model " + url + "\n")
+        meta = requests.head(url)
+        dsize = "{:,}".format(int(meta.headers.get("content-length")))
+        if not args.quiet: print("Downloading '{}' ({} bytes) ...\n".
+                                 format(mlmfile, dsize))
+
+        # Download the archive from the URL.
+
+        try:
+            urllib.request.urlretrieve(url, local)
+        except urllib.error.HTTPError as error:
+            msg = "{}'{}' {}."
+            msg = msg.format(APPX, url, error.reason.lower())
+            print(msg, file=sys.stderr)
+            sys.exit(1)
+
     zip = zipfile.ZipFile(local)
     zip.extractall(MLINIT)
 
@@ -255,18 +293,17 @@ def install_model(args):
             for f in files:
                 tfilename = os.path.join(pth, f)
                 dsz += os.path.getsize(tfilename)
-        print("Extracted '{}' into\n'{}' ({:,} bytes).\n".
+        print("Extracted '{}' into\n'{}' ({:,} bytes).".
               format(mlmfile, path, dsz))
             
     # Suggest next step. README or DOWNLOAD
     
     if not args.quiet:
-        msg = "Read the model information with:\n\n  $ {} readme {}\n"
-        msg = msg.format(CMD, model)
-        print(msg)
+        utils.print_next_step('install', model=model)
     
 #-----------------------------------------------------------------------
 # DOWNLOAD
+#------------------------------------------------------------------------
 
 def download_model(args):
     """Download the large pre-built model."""
@@ -283,12 +320,11 @@ def download_model(args):
     utils.check_model_installed(model)
     
     if not args.quiet:
-        msg = "Model information is available:\n\n  $ {} readme {}\n"
-        msg = msg.format(CMD, model)
-        print(msg)
+        utils.print_next_step('download', model=model)
 
-#-----------------------------------------------------------------------
+#------------------------------------------------------------------------
 # README
+#------------------------------------------------------------------------
 
 def readme(args):
     """Display the model's README information."""
@@ -306,14 +342,12 @@ def readme(args):
     # Display the README.
     
     with open(readme, 'r') as f:
-        print(f.read())
+        print(utils.drop_newline(f.read()))
     
     # Suggest next step.
 
     if not args.quiet:
-        msg = "Model dependencies are listed using:\n\n  $ {} configure {}\n"
-        msg = msg.format(CMD, model)
-        print(msg)
+        utils.print_next_step('readme', model=model)
 
 #------------------------------------------------------------------------
 # LICENSE
@@ -326,10 +360,13 @@ def license(args):
     
 #-----------------------------------------------------------------------
 # COMMANDS
+#------------------------------------------------------------------------
 
 def list_model_commands(args):
     """ List the commands supported by this model."""
 
+    # Setup.
+    
     model = args.model
 
     # Check that the model is installed.
@@ -338,22 +375,30 @@ def list_model_commands(args):
     
     info = utils.load_description(model)
 
-    msg = "The model '{}' ({}) supports the following commands:"
-    msg = msg.format(model, info['meta']['title'])
-    msg = textwrap.fill(msg, width=60)
-    print(msg + "\n")
-    
-    yaml.dump(info['commands'], sys.stdout, default_flow_style = False)
+    msg = "The model '{}' "
+    if 'title' not in info['meta']:
+        title = None
+    else:
+        title = utils.lower_first_letter(utils.dropdot(info['meta']['title']))
+        msg += "({}) "
+
+    msg += "supports the following commands:"
+    msg = msg.format(model, title)
+    msg = textwrap.fill(msg, width=75)
+    print(msg)
+
+    for c in info['commands']:
+        print("\n$ {} {} {}".format(CMD, c, model))
+        print("  " + info['commands'][c])
 
     # Suggest next step.
     
     if not args.quiet:
-        msg = "Model dependencies are listed using:\n\n  $ {} configure {}\n"
-        msg = msg.format(CMD, model)
-        print(msg)
+        utils.print_next_step('commands', description=info, model=model)
 
 #-----------------------------------------------------------------------
 # CONFIGURE
+#------------------------------------------------------------------------
 
 def configure_model(args):
     """Ensure the user's environment is configured."""
@@ -400,49 +445,40 @@ def configure_model(args):
 
     utils.check_model_installed(model)
 
-    # If there is a configure script and this is a Ubuntu host, then
-    # run the configure script.
+    # If there are any configure scripts then run them, else print the
+    # list of supplied dependencies if any. Note that Python's 'or' is
+    # lazy evaluation.
 
-    conf = os.path.join(path, "configure.sh")
-    if platform.dist()[0] in set(['debian', 'Ubuntu']) and os.path.exists(conf):
-        command = "bash configure.sh"
-        if not args.quiet:
-            msg = "Configuration will take place using '{}'.\n"
-            msg = msg.format(conf)
-            print(msg)
-        proc = subprocess.Popen(command, shell=True, cwd=path, stderr=subprocess.PIPE)
-        output, errors = proc.communicate()
-        if proc.returncode != 0:
-            print("An error was encountered:\n")
-            print(errors.decode("utf-8"))
-    else:
-        # For now simply list the declared dependencies for the user to
-        # make sure they have it all installed.
+    conf = utils.configure(path, "configure.sh", args.quiet)
+    conf = utils.configure(path, "configure.R", args.quiet) or conf
+    conf = utils.configure(path, "configure.py", args.quiet) or conf
 
-        if not args.quiet:
-            msg = """
-Configuration is yet to be automated. The following dependencies are required:
-"""
-            print(msg)
+    if not conf:
+        try:
+            info = utils.load_description(model)
+            deps = info["meta"]["dependencies"]
 
-        info = utils.load_description(model)
-        msg = info["meta"]["dependencies"]
-
-        print("  ====> \033[31m" + msg + "\033[0m")
+            if not args.quiet:
+                msg = "No configuration script provided for this model. "
+                msg = msg + "The following dependencies are required:\n"
+                print(msg)
 
+            print("  ====> \033[31m" + deps + "\033[0m")
+        except:
+            print("No configuration provided (maybe none is required).")
+            
     # Suggest next step.
     
     if not args.quiet:
-        msg = "\nOnce configured run the demonstration:\n\n  $ {} demo {}\n"
-        msg = msg.format(CMD, model)
-        print(msg)
+        utils.print_next_step('configure', model=model)
 
 #-----------------------------------------------------------------------
 # DISPATCH
+#------------------------------------------------------------------------
 
 def dispatch(args):
     """Dispatch other commands to the appropriate model provided script."""
-    
+
     cmd   = args.cmd
     model = args.model
     path  = MLINIT + model
@@ -454,27 +490,52 @@ def dispatch(args):
     utils.check_model_installed(model)
     
     desc = utils.load_description(model)
+
+    # Check if cmd needs to use graphic display indicated in DESCRIPTION.yaml.
+
+    if 'display' in desc['meta'] and cmd in desc['meta']['display'] and os.environ.get('DISPLAY', '') == '':
+        msg = "Graphic display is required but not available for command '{}'. Continue [y/N]? "
+        msg = msg.format(cmd)
+        sys.stdout.write(msg)
+        choice = input().lower()
+        if choice != 'y':
+            msg = """
+To enable DISPLAY be sure to connect to the server using 'ssh -X'
+or else connect to the server's desktop using a local X server like X2Go.
+
+"""
+            sys.stdout.write(msg)
+            sys.exit(1)
+
+    # Obtain the default/chosen language for the package.
+
+    lang = desc["meta"]["languages"]
         
     # Obtain the specified script file.
     
-    script  = desc["commands"][cmd]["script"].split(" ")[0] + " " + param
+    script  = cmd + "." + lang
+    
+    if args.debug:
+        print(DEBUG + "execute the script: " + os.path.join(path, script))
+     
+    if not os.path.exists(os.path.join(path, script)):
+        msg = """{}The command '{}' was not found for this model.
+
+Try using 'commands' to list all supported commands:
+
+  $ {} commands {}
+""".format(APPX, cmd, CMD, model)
+        print(msg, file=sys.stderr)
+        sys.exit(1)
 
     # Determine the interpreter to use
     #
     # .R => Rscript; .py => python, etc.
 
-    (root, ext) = os.path.splitext(script)
-    ext = ext.strip()
-    if ext == ".R":
-        interpreter = "Rscript"
-    elif ext == ".py":
-        interpreter = "python3"
-    else:
-        msg = "Could not determine an interpreter for extension '{}'".format(ext)
-        print(msg, file=sys.stderr)
-        sys.exit()
+    interpreter = utils.interpreter(script)
+
+    command = "{} {} {}".format(interpreter, script, param)
 
-    command = "{} {}".format(interpreter, script)
     if args.debug:
         print(DEBUG + "(cd " + path + "; " + command + ")")
     proc = subprocess.Popen(command, shell=True, cwd=path, stderr=subprocess.PIPE)
@@ -482,6 +543,11 @@ def dispatch(args):
     if proc.returncode != 0:
         print("An error was encountered:\n")
         print(errors.decode("utf-8"))
+    else:
+        # Suggest next step
+
+        if not args.quiet:
+            utils.print_next_step(cmd, description=desc, model=model)
     
 #------------------------------------------------------------------------
 # DONATE
@@ -518,8 +584,14 @@ def remove_model(args):
     
     model  = args.model
     if model is None:
-        path = MLINIT
-        msg  = "*Completely* remove all installed models in '{}' [y/N]? "
+        if os.path.exists(MLINIT):
+            path = MLINIT
+            msg  = "*Completely* remove all installed models in '{}' [y/N]? "
+        else:
+            msg = "The local model folder '{}' does not exist. Nothing to do."
+            msg = msg.format(MLINIT)
+            print(msg)
+            sys.exit(1)
     else:
         path = MLINIT + model
         msg = "Remove '{}' [y/N]? "
@@ -530,5 +602,8 @@ def remove_model(args):
 
     sys.stdout.write(msg.format(path))
     choice = input().lower()
-    if choice == 'y': rmtree(path)
-
+    if choice == 'y':
+        rmtree(path)
+    else:
+        if model is None and not args.quiet:
+            utils.print_next_step('remove')
